{% load i18n %}
{% if form %}
<form method="post" enctype="multipart/form-data" action="{{ form_url }}" class="add-permission">
    <input type="hidden" name="next" value="{{ next }}"/>
    {{ form.as_p }}
    <p class="submit">
    {% if approved %}
        <input type="submit" value="{% trans "Grant permission" %}"/>
    {% else %}
        <input type="submit" value="{% trans "Request permission" %}"/>
    {% endif %}
    </p>
</form>
{% endif %}
